### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Any2TitleAdapter class for the Zope 3 based ng.adapter.title package

$Id: product.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Yegor Shershnev, 2007"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"


from zope.component import adapts
from titleadapterbase import TitleAdapterBase
from zope.interface import Interface


class Any2TitleAdapter(TitleAdapterBase) :

    adapts(Interface)
    
    @property
    def title(self) :
        return self.context.__class__.__name__



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""NameChooser adapter

$Id: namechooser.py 50637 2008-02-12 21:29:19Z cray $
"""
__author__  = "Andrey Orlov"
__license__ = "GPL"
__version__ = "$Revision: 50637 $"

from zope.interface import Interface
from ng.adapter.title.interfaces import ITitle
import re
match = re.compile("^(?P<name>.*[^0-9])(?P<number>[0-9]+)$").match
                
class NameChooser(object) :

    def __init__(self,context) :
        self.context = context

    def checkName(self, name, object) :
        """Check whether an object name is valid.
            Raises a user error if the name is not valid. """
        return True
     
    def chooseName(self, name, object) :
        """ Choose a unique valid name for the object
            The given name and object may be taken into account when choosing the name. """

        if name :
            title = name
        else :            
            try :
                title = ITitle(object).title
            except TypeError,msg :
                print "Can't get title becouse of",msg
                title = object.__class__.__name__

        while title in self.context :
            res = match(title)
            if res :
                d = res.groupdict()
                d['number']=int(d['number'])+1
                title = "%(name)s%(number)u" % d
            else :                
                title = title + "0"
                                    
        return title            



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""The base adapter 

$Id: anytitilesubscriberbase.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Elena Antusheva, 2008"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"


from interfaces import IAnyTitle
from zope.interface import implements

class AnyTitleSubscriberBase(object) :

    implements(IAnyTitle)
    order = 0

    def __init__(self,context) :
        self.context = context

    def __cmp__(self,x) :
        return cmp(self.order,x.order)
        


### -*- coding: utf-8 -*- #############################################
#######################################################################
""" Any Interface Adapter to IPager Interface

$Id: pageradapter.py 50637 2008-02-12 21:29:19Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 50637 $"

from zope.interface import implements,Interface
from zope.component import adapts
from ng.app.registry.interfaces import IRegistry
from zope.app.zapi import getUtility
from zope.cachedescriptors.property import CachedProperty
from interfaces import IPager, IPagerSource
from zope.publisher.interfaces.http import IHTTPRequest
from ng.adapter.requestext.interfaces import IRequestExt
from zope.traversing.browser import absoluteURL

class PagerAdapter(object):
    __doc__ = __doc__

    implements(IPager)
    adapts(Interface,IHTTPRequest)
    
    orphan = 3
    size = 10
    revert = False
    _absolute_url = None

    def __init__(self, context, request):
        self.context = IPagerSource(context)
        self.request = request
        self.requestext = IRequestExt(request)
        self.requestext.setParameters(exclude=('current',))

    def setPagerParameters(self,revert=None,orphan=None,size=None,context=None) :
        if context :
            self._absolute_url = absoluteURL(context,self.request)
        if revert :
            self.revert = revert 
        if orphan :
            self.orphan = orphan 
        if size :
            self.size = size 
        try :            
            IRegistry(self.context).export(self,'orphan','size','revert')
        except TypeError :
            pass

    def getOb(self,key) :
        return self.context[key]
        
    def getKey(self,index) :
        return self.keys[index]

    @CachedProperty
    def absolute_url(self) :
        return str(self.request.URL)
    
        if self._absolute_url :
            return self._absolute_url
        else :
            absoluteURL(self.context,self.request)                    


    @CachedProperty
    def keys(self) :
        res = self.context.keys()
        if self.revert :
            res.reverse()
        return res            
         
    @CachedProperty
    def index(self) :
        try :
            curr = self.request.form['current']
        except KeyError :
            return 0
                    
        return self.keys.index(curr)

    @CachedProperty
    def beforeURLs(self) :
        for item in self.befores :
            yield self.absolute_url+'?'+self.requestext.query(current=item)
            

    @CachedProperty
    def befores(self) :
        for item in xrange(0,self._before,self.size-self.orphan) :
            yield self.getKey(item)
            

    @CachedProperty
    def have_before(self) :
        try :    
            return self._before < self.index
        except Exception,msg :
            print "RRRR",msg            
            
        try :
            self._before
        except IndexError :
            return False
        except Exception,msg:
            print "qq1",msg
        else :
            return True                        

    @CachedProperty
    def beforeURL(self) :
        return self.absolute_url+'?'+self.requestext.query(current=self.before)

    @CachedProperty
    def before(self) :
        return self.getKey(self._before)
        
    @CachedProperty
    def _before(self) :
        return max(self.index + self.orphan - self.size,0)
                
    @CachedProperty
    def chunk(self) :
    
        for item in xrange(self.index,min(self.index+self.size,self.len),1) :
            yield self.getOb(self.getKey(item))
            
    @CachedProperty
    def have_after(self) :
        return self._after < self.len-self.orphan
            
    @CachedProperty
    def afterURL(self) :
        return self.absolute_url+'?'+self.requestext.query(current=self.after)

    @CachedProperty            
    def after(self) :
        return self.getKey(self._after)
        
    @CachedProperty            
    def _after(self) :
        return min(self.index + self.size-self.orphan,self.len-1)

    @CachedProperty
    def afterURLs(self) :
        for item in self.afters :
            yield self.absolute_url+'?'+self.requestext.query(current=item)
            
    @CachedProperty
    def afters(self) :
        for item in xrange(self._after,self.len-1,self.size-self.orphan) :
            yield self.getKey(item)
    
    @CachedProperty
    def len(self) :
        return len(self.keys)

        


# -*- coding: utf-8 -*-
"""The nsinterface class.

$Id: nsinterface.py 50637 2008-02-12 21:29:19Z cray $
"""
__author__  = "Dima Khozin & Andrey Orlov"
__license__ = "GPL"
__version__ = "$Revision: 50637 $"
__date__ = "$Date: 2008-02-13 00:29:19 +0300 (Срд, 13 Фев 2008) $"
__credits__ = """Andrey Orlov, for idea and common control"""

from zope.interface import implements
from zope.traversing.interfaces import ITraversable
from zope.component.interface import nameToInterface
from zope.traversing.namespace import SimpleHandler
from zope.security.proxy import removeSecurityProxy
from zope.traversing.interfaces import IPhysicallyLocatable
from zope.location import location

class Nsinterface(SimpleHandler):
    """ ++interface++"""

    implements(ITraversable)

    def traverse(self,name,ignored) :
        """алгоритм приведения контекстного объекта к нужному интерфейсу"""
        res = nameToInterface(self.context,name)(self.context)
        res = removeSecurityProxy(res)
        if True : #not IPhysicallyLocatable(res, False):
            res = location.LocationProxy(
                res, self.context, '++nsinterface++'+name)
            return res
        return res
        



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""TitleAdapterBase class for the Zope 3 based ng.adapter.title package

$Id: product.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Yegor Shershnev, 2007"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"

from interfaces import ITitle
from zope.interface import implements

class TitleAdapterBase(object) :

    implements(ITitle)
    
    def __init__(self, context) :
        self.context = context



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Interfaces for the Zope 3 based product package

$Id: interfaces.py 50637 2008-02-12 21:29:19Z cray $
"""
__author__  = "Elena Antusheva, 2008"
__license__ = "GPL"
__version__ = "$Revision: 50637 $"
 
from zope.interface import Interface

from zope.schema import Text, TextLine, Field, Bool, Datetime
from zope.app.container.interfaces import IContained, IContainer
from zope.app.container.constraints import ItemTypePrecondition
from zope.app.container.constraints import ContainerTypesConstraint
                
class IAnyTitle(Interface) :
    
    title = TextLine(title = u'Title')
    shorttitle = TextLine(title = u'Title')



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Object size interface

$Id: interfaces.py 50637 2008-02-12 21:29:19Z cray $
"""

__author__  = "Andrey Orlov, 2008"
__license__ = "GPL"
__version__ = "$Revision: 50637 $"

from zope.interface import Interface
from zope.schema import Int, Field

class IRecordSize(Interface):
    """Record Size interface"""

    size = Int(
        title=u"Size",
        description=u"Size of database record",
        default=0,
        required=True)





### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Interfaces for the Zope 3 based ng.adapter.path package

$Id: interfaces.py 50637 2008-02-12 21:29:19Z cray $
"""
__author__  = "Yegor Shershnev, 2007"
__license__ = "GPL"
__version__ = "$Revision: 50637 $"
 
from zope.interface import Interface

from zope.schema import TextLine, Tuple


class IPath(Interface) :
    """ Interface IPath.

        path - path from root object of Zope to current object
        
        titledpath - path from root object of Zope to current object,
            contains from titles ob objects
    """
    
    path = TextLine(title=u"Path",
                    description=u"Path from root object of Zope to current object")
    
    titledpath = TextLine(title=u"Titled path",
                          description=u"Titled path from root object of Zope to current object")

    tailpath = Tuple()



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""
Interfaces for the Zope 3 based ng.adapter.куйгуые package

$Id: interfaces.py 50637 2008-02-12 21:29:19Z cray $
"""
__author__  = "Andrey Orlov"
__license__ = "GPL"
__version__ = "$Revision: 50637 $"

from zope.interface import Interface

class IRequestExt(Interface):

    def keys(exclude=(),include=None,use_include=None):
        """ Вернуть отфильтрованный список ключей запроса

        include - если не равно None и флаг use_include не равен 0, то в
                список войдут только перечисленные в этом атрибуте ключи,
        exclude - перечисленные ключи не войдут в список,
        use_include - если равен 0, то параметр include не будет
                использован.
        """

    def dict(exclude=[],include=None,use_include=None):
        """ Вернуть элементы запроса, отфильтрованные keys и
        преобразованные в словарь имя : значения. Элементы типа record отдаются
        поэлементно как ключи вида <элемент>.<атрибут>.  Парамтры см.
        keys.  """

    def list(exclude=[], include=None, use_include=None):
        """ Вернуть элементы запроса, отфильтрованные keys и
        преобразованные в список словарей. Параметры см. keys. """

    def hidden(exclude=[],include=None,use_include=None) :
        """ Вернуть элементы запроса, отфильтрованные keys и
        преобразованные в элементы ввода HIDDEN, парамтры см.
        keys
        """

    def query(exclude=[],include=None,use_include=None) :
        """ Вернуть строку query, построенную из элементов запроса,
        возвращенных keys
        """

    def pack_dict(**kw):
        """ Вернуть упакованный методом pack хэш аргументов """

    def pack_unpack_dict(**kw):
        """ Вернуть результат распаковки query методом unpack. Если
        типа результата - словарь, то его ключи копируются в словарь form.
        """

    def pack(item):
        """ Вернуть параметр item, упакованный для передачи через запрос.
        Обычный метод упаковки - marshal + zip + base64 + "нормализация"
        base64.
        """

    def unpack(query):
        """ Вернуть распакованое значение параметра query, ранее
        запакованное методом pack
        """



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""Contained2PathAdapter class for the Zope 3 based ng.adapter.path package

$Id: product.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Yegor Shershnev, 2007"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"


from zope.interface import implements
from zope.app.container.interfaces import IContained
from interfaces import IPath
from zope.component import adapts
from ng.adapter.title.interfaces import ITitle
from pathadapterbase import PathAdapterBase


class Contained2PathAdapter(PathAdapterBase) :

    implements(IPath)
    adapts(IContained)
   
    @property
    def titledpath(self) :
        return "/".join([
            IPath(IContained(self.context).__parent__).titledpath,
            ITitle(self.context).title])
    
    @property
    def path(self) :
        return "/".join([IPath(IContained(self.context).__parent__).path, self.name])

    @property    
    def name(self) :
        return IContained(self.context).__name__
        
    def _tailpath(self,path) :
        next = IPath(IContained(self.context).__parent__)
        return next._tailpath(
            path + ("/".join((next.name,path[-1])),)
        )



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""The adapter ZDC2TitleSubscribe that adopt IAttributeAnnotatable
   interface to ITitle interface

$Id: zdc2titlesubscribe.py 12897 2007-11-10 15:32:08Z cray $
"""
__author__  = "Elena Antusheva, 2008"
__credits__  = "Yegor Shershnev"
__license__ = "GPL"
__version__ = "$Revision: 12897 $"

from zope.component import adapts
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.dublincore.interfaces import IZopeDublinCore
from anytitlesubscriberbase import AnyTitleSubscriberBase


class ZDC2TitleSubscribe( AnyTitleSubscriberBase ) :
    order = 1
    adapts(IAttributeAnnotatable)
    
    @property
    def title(self) :
        return IZopeDublinCore(self.context).title


